const state = {
    suppliers: [],
    loadingSuppliers: true,
    filteredSuppliers: [],
    selectedSupplier: null
}

const getters = {
    /**
     * 
     * @param {*} state 
     */
    suppliers(state) {
        return state.suppliers
    },
    /**
     * 
     * @param {*} state 
     */
    loadingSuppliers(state) {
        return state.loadingSuppliers
    },
    /**
     * 
     * @param {*} state 
     */
    visibleSuppliers(state) {
        return state.filteredSuppliers.length === 0 ? state.suppliers : state.filteredSuppliers
    },
    /**
     * 
     * @param {*} state 
     */
    selectedSupplier(state) {
        return state.selectedSupplier
    }
}

const mutations = {
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setSuppliers(state, payload) {
        state.suppliers = [...payload]
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */ 
    setLoadingSuppliers(state, payload) {
        state.loadingSuppliers = !!payload
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setSelectedSupplier(state, payload) {
        state.selectedSupplier = payload
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setFilteredSuppliers(state, payload) {
        state.filteredSuppliers = [...payload]
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    addSupplier(state, payload) {
        state.suppliers = [payload, ...state.suppliers]
    }
}

const actions = {
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    initSuppliers({ commit }, payload) {
        commit('setSuppliers', payload)
        commit('setLoadingSuppliers', false)
    },
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    selectSupplier({ commit }, payload) {
        commit('setSelectedSupplier', payload)
    },
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    filterSuppliers({ commit }, payload) {
        commit('setFilteredSuppliers', payload)
    },
    /**
     * 
     * @param {*} param0 
     */
    submitSupplier({ commit, dispatch }, payload) {
        if (payload) {
            if (!payload.name || !payload.ruc || !payload.phone) {
                dispatch('notify', 'Complete los campos para guardar')
                return
            }

            commit('setLoadingSuppliers', true)
            dispatch('createSupplier', payload)
        }

        dispatch('hideSupplierForm')
    },
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    addSupplier({ commit }, payload) {
        commit('addSupplier', payload)
        commit('setLoadingSuppliers', false)
    },
    /**
     * 
     * @param {*} param0 
     */
    resetSupplier({ commit }) {
        commit('setLoadingSuppliers', true)
        commit('setSuppliers', [])
        commit('setFilteredSuppliers', [])
        commit('setSelectedSupplier', null)
    }
}

export default {
    state,
    getters,
    mutations,
    actions
}